package com.openapi.sdk.util;

import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.conn.ssl.NoopHostnameVerifier;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
import org.apache.http.ssl.SSLContextBuilder;
import org.apache.http.util.EntityUtils;

import javax.net.ssl.SSLContext;
import java.nio.charset.StandardCharsets;

@Slf4j
public class HttpsUtils {
    private static final int MAX_TOTAL = 800;
    private static final int MAX_PER_ROUTE = 20;
    private static final int CONNECT_TIMEOUT = 5000;
    private static final int READ_TIMEOUT = 5000;

    private static PoolingHttpClientConnectionManager connManager;
    private static CloseableHttpClient httpClient;

    static {
        try {
            // Create a connection manager with connection pool
            connManager = new PoolingHttpClientConnectionManager();
            connManager.setMaxTotal(MAX_TOTAL);
            connManager.setDefaultMaxPerRoute(MAX_PER_ROUTE);

            // Create an HttpClient with the connection manager
            SSLContext sslContext = new SSLContextBuilder()
                    .loadTrustMaterial(null, (certificate, authType) -> true)
                    .build();

            httpClient = HttpClients.custom()
                    .setSSLContext(sslContext)
                    .setSSLHostnameVerifier(NoopHostnameVerifier.INSTANCE)
                    .setConnectionManager(connManager)
                    .build();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static String doPost(String url, int connTimeout, int readTimeout) throws Exception {
        return doPost(url, "", connTimeout, readTimeout);
    }

    public static String doPost(String url, String param, int readTimeout, int connectTimeout) throws Exception {
        log.info("====>进入自定义HttpsUtils doPost" );

        RequestConfig requestConfig = RequestConfig.custom()
                .setConnectTimeout(connectTimeout)
                .setSocketTimeout(readTimeout)
                .build();

        HttpPost httpPost = new HttpPost(url);
        httpPost.setConfig(requestConfig);
        httpPost.setHeader("Content-Type", "application/json; charset=utf-8");
        httpPost.setEntity(new StringEntity(param, StandardCharsets.UTF_8));

        HttpResponse response = httpClient.execute(httpPost);
        HttpEntity entity = response.getEntity();
        if (entity != null) {
            return EntityUtils.toString(entity, StandardCharsets.UTF_8);
        }
        return "";
    }

    public static void main(String[] args) throws Exception {
            String url = "https://www.baidu.com";
            String result = HttpsUtils.doPost(url, 5000, 5000);
            System.out.println(result);
    }
}